// ===============================================================
// CBP_1421_Helper1a.c
// 31-12-20
//
// ===============================================================

#include <stdio.h>
#include <webots/robot.h>
#include <webots/motor.h> 
#include <webots/distance_sensor.h>
#include <webots/light_sensor.h>
  
WbDeviceTag ps[8];
WbDeviceTag ls[8];
char ps_names[8][4] = {"ps0", "ps1", "ps2", "ps3", "ps4", "ps5", "ps6", "ps7"};
char ls_names[8][4] = {"ls0", "ls1", "ls2", "ls3", "ls4", "ls5", "ls6", "ls7"};
WbDeviceTag left_motor, right_motor;
WbDeviceTag toflight;


typedef struct { double x; double y; } LUT;

// Function to invert the LUT of the distance sensor, to give an
// output distance
double lookUpDistance(LUT *c, double x) {
    int i;
    int n = 10;
    for( i = 0; i < n-1; i++ ) {
        if ( c[i].x <= x && c[i+1].x >= x ) {
            double diffx = x - c[i].x;
            double diffn = c[i+1].x - c[i].x;
            return c[i].y + ( c[i+1].y - c[i].y ) * diffx / diffn; 
        }
    }
    return -1; // Not in Range
}

//typedef struct { double x; double y; } LUT;
LUT c[10] = {
//  {30, 0.5},   // Added CBP
  {67.19, 0.07},
  {104.09, 0.06},
  {120.0, 0.05},
  {158.03, 0.04},
  {234.93, 0.03},
  {383.84, 0.02},
  {601.46, 0.015},
  {1465.63, 0.01},
  {2133.33, 0.005},
  {4095.00, 0.00}
};


// Device initialization -----------------------------------------

void cbp_motors_init() {
 // get a handler to the motors and set target position to infinity (speed control).
  left_motor = wb_robot_get_device("left wheel motor");
  right_motor = wb_robot_get_device("right wheel motor");
  wb_motor_set_position(left_motor, INFINITY);
  wb_motor_set_position(right_motor, INFINITY);
  wb_motor_set_velocity(left_motor, 0.0);
  wb_motor_set_velocity(right_motor, 0.0);

}

void cbp_sensors_init(int timeStep) {
  for (int i = 0; i < 8; i++) {
    ps[i] = wb_robot_get_device(ps_names[i]);
    wb_distance_sensor_enable(ps[i], timeStep);
    ls[i] = wb_robot_get_device(ls_names[i]);
    wb_light_sensor_enable(ls[i], timeStep);    
  }    
  toflight = wb_robot_get_device("tof");
  wb_distance_sensor_enable(toflight,timeStep);	
		
}	

void cbp_motors_set_velocity(double omegaL, double omegaR) {
    wb_motor_set_velocity(left_motor,  omegaL);
    wb_motor_set_velocity(right_motor, omegaR);	
}
	
void cbp_get_distance_vals(double vals[]) {
  for (int i = 0; i < 8; i++) {
    double ps_val = wb_distance_sensor_get_value(ps[i]);      
	vals[i] = lookUpDistance(c,ps_val);	  
  }	
}	

double cbp_get_timeOfFlight_vals(){
   return(wb_distance_sensor_get_value(toflight)); 
}

double invertLightSensorVals(double val) {
  double output = 4095.0 - val;
  if(output < 0.0) output = 0.0;
  return output;
}

void cbp_get_lightsensor_vals(double vals[]) {
  for (int i = 0; i < 8; i++) {
    double ls_val = wb_light_sensor_get_value(ls[i]);      
	vals[i] = invertLightSensorVals(ls_val);	  
  }	
}

// Function provides a non-blovking delay ------------------------
bool delay(double delay) {
  static double previous_time;
  static bool done = false;  
  static bool first_pass = true;
  
  if(first_pass){
    previous_time =  wb_robot_get_time();
    first_pass = false;
  }
      
  double current_time = wb_robot_get_time();
  //printf("     prev %f  curr = %f\n",previous_time,current_time);
    
  if(current_time - previous_time > delay) {
    previous_time = current_time;
    done = true;
    first_pass = true;
   // printf("\n");
  } else {
    done = false;  
   // printf(".");
  }
  return done;
}

// Function to provide a blocking delay ----------------------------
void blocking_delay(double seconds, int time_step) {
  const double ms = seconds * 1000.0;
  int elapsed_time = 0;
  while (elapsed_time < ms) {
    wb_robot_step(time_step);
    elapsed_time += time_step;
	printf(".");
  }
  printf("\n");
}
