// ===================================================================
//
// CBP_1421_Sensorium_1.c 
//
// 30_12_20 Devel
//
// Simple controller to explore distance, light and time of flight.
//
// ==================================================================
 
// Webots headers 
#include <webots/robot.h>

// C-function headers   
#include <stdio.h>

// Headers for this module. Helper_1 provides calls to distance sensor,
// light sensor and time-of-flight sensor. Als motor drive function.
#include "CBP_1421_Helper1.h"
  
// Robot Parameters Speed in rad/sec. Distances in metres.
// Time step is abnormally large to slow down data output
#define TIME_STEP 500    
#define WHEEL_RADIUS 0.0205
#define AXLE_LENGTH 0.053
#define MAX_SPEED 6.28

// Global variables
double omegaL, omegaR;
double dist_vals[8];
double light_vals[8];

int main(int argc, char **argv) {

  wb_robot_init();

  cbp_motors_init();
  cbp_sensors_init(TIME_STEP);
   
  while (wb_robot_step(TIME_STEP) != -1) {
          
    // Distance Sensors
    cbp_get_distance_vals(dist_vals);
    // Output actual vals in metres. -1 means out of range  
    printf("Distance ");
    for(int i=0;i<8;i++) {
      printf("%+5.3f  ",dist_vals[i]);  
    }
    printf("\n");
    
    // Light Sensors
    printf("Light     ");    
    cbp_get_lightsensor_vals(light_vals);
    for(int i=0;i<8;i++) {
      printf("%+8.2f  ",light_vals[i]);  
    }
    printf("\n");
         
    // Time of flight
    printf("TOF       ");      
    double dist = cbp_get_timeOfFlight_vals();
    printf("%+8.2f\n",dist);  
    printf("..\n");
    
    // Do not drive robot
    omegaL = 0.0;
    omegaR = 0.0;
    cbp_motors_set_velocity(omegaL,omegaR);

  };

  wb_robot_cleanup();

  return 0;
}
